import { csv, range, timeParse } from 'd3';

/* Parses and formats the time to a single year. */
export const parseYear = timeParse('%Y')

/* Checks whether the string is in uppercase. */
const allCaps = str => str === str.toUpperCase();

/* Checks whether the string is in uppercase & is not "WORLD". */
const isRegion = name => allCaps(name) && name !== 'WORLD';

/* "Melts" data from row to column. 
1. The first forEach loop maps each row and stores the country name in
the variable "name".
2. Loop over a range year in a inner for loop and retrieves
the population data.
3. Merge these data and append it as a single row to the return
value. 
*/
const melt = (unData, minYear, maxYear) => {
  const years = range(minYear, maxYear + 1);
  const data = [];

  unData.forEach(d => {
    const name = d['Region, subregion, country or area *']
    	.replace('AND THE CARIBBEAN', '');
    years.forEach(year => {
      const population = +d[year].replace(/ /g, '') * 1000;
      const row = {
        year: parseYear(year),
        name,
        population
      }
      data.push(row);
    })
  })

  return data.filter(d => isRegion(d.name));
}

/* Load data from two files, process and concat them,
and return the results to index.js. */
export const loadAndProcessData = () => 
	Promise
	.all([csv('https://vizhub.com/curran/datasets/un-population-estimates-2017-medium-variant.csv'),
        csv('https://vizhub.com/curran/datasets/un-population-estimates-2017.csv'),
      ])
  .then( ([unDataMediumVariant, unDataEstimates]) => {
    
    return melt(unDataEstimates, 1950, 2014)
    	.concat(melt(unDataMediumVariant, 2015, 2100));

  });

